/* Each LED is wrapped in a Table Row (TR) element (Ref-TR). Each TR
   element contains 3 TD elements and the 3 'mk..' (for make) functions
   below create the HTML that goes into these 3 TD elements.
*/

/* TD 1: the LED is simply a DIV element styled by using the CSS from
   the following page: http://cssdeck.com/labs/css-leds
*/
function mkLight(ptid,ledId,color,on) {
    return '<td class="light-status"><div id="led-'+ptid+ledId+'" class="bulb led-'+
        color+(on ? '' : ' led-off')+'"></div></td>';
}

/* TD 2: the LED name */
function mkLightName(name) {
    return '<td>'+name+'</td>';
}

/* TD 3: the LED on/off switch is a standard HTML checkbox that is
   styled using CSS3 and by using CSS3 transformation. We used the
   on/off FlipSwitch generator on the following page for generating
   the CSS: https://proto.io/freebies/onoff/
*/
function mkLightSwitch(ptid,ledId,on) {
    var x =
        '<td>'+
        '<div class="onoffswitch">'+
        '<input type="checkbox" name="onoffswitch" class="onoffswitch-checkbox" id="switch-'+
        ptid+'-'+ledId+'" '+(on ? "checked" : "")+'/>'+
        '<label class="onoffswitch-label" for="switch-'+ptid+'-'+ledId+'">'+
        '<span class="onoffswitch-inner"></span>'+
        '<span class="onoffswitch-switch"></span>'+
        '</label>'+
        '</div>'+
        '</td>';
    return x;
}

function setInfo(info) {
    $("#devices").html('<tr><td></td><td>'+info+'</td></tr>');
};


$(function() {

    var SMSG_SERVER_HELLO=1
    var SMSG_BROWSER=3;
    var SMSG_DEVICE_INFO=4;
    var SMSG_SET_LIGHT=5;
    var SMSG_COMMAND_LIST=6;
    var SMSG_EXEC_COMMAND=7;

    var serverTID; // The SMQ client application in the server.

    /* Check if the browser supports WebSockets */
    if(! SMQ.websocket() ) {
        setInfo('<h2>Your browser stinks!</h2>');
        return; /* Stop: no support */
    }

    // Create a SimpleMQ instance and connect to the broker.
    var smq = SMQ.Client(SMQ.wsURL(window.location.pathname));

    /* We use the onmsg as a "catch all" for non managed messages we
       receive. This function should not be called since we install
       callbacks for all subscribed events. The onmsg is typically
       used for error checking during development.
    */
    smq.onmsg=function(data,ptid,tid,subtid) {
        console.log("Received unexpected data:"+data+", from "+ptid+
                    ", tid="+tid+", subtid="+subtid);
        console.log("  data string:"+SMQ.utf8.decode(data))
    };

    /* The disconnect callback function removes all devices and
       LEDS. A disconnect message is shown. The function instructs the
       SimpleMQ client stack to attempt to reconnect after 3
       seconds. This function will be called repeatedly if the
       SimpleMQ client stack is unable to reconnect.
     */
    smq.onclose=function(message,canreconnect) {
        setInfo('<h2>Disconnected!</h2><p>'+
                         (message ? message : '')+'</p>');
        $("#commands").hide();
        if(canreconnect) return 3000;
    };

    var lightBanks; // Number of connected devices

    /* We redirect the onconnect event to our function. The function
       replaces the initial "Connecting..." message.
     */
    smq.onconnect=function() {
        lightBanks=0;
        setInfo('<h3>No light controller connected</h3>');
    }

    /* Called after a disconnect (smq.onclose) and after a re-connect
       attempt succeeded.
    */
    smq.onreconnect=smq.onconnect

    smq.subscribe("self", SMSG_SERVER_HELLO, {"onmsg":function(data,ptid) {
        serverTID=ptid;
        smq.publish("", serverTID, SMSG_BROWSER);
    }});

    var hasCommandList=false;
    function commandList(list) {
        if(hasCommandList) return;
        hasCommandList=true;
        for(var i=0 ; i < list.length; i++) {
            $("#commands").append('<li><a cmd="'+list[i].cmd+
                                  '" href="#" class="redButton">'+
                                  list[i].name+'</a></li>');
        }
        $("#commands a").click(function(ev) {
            var cmd = $(this).attr('cmd');
            smq.publish(cmd,serverTID,SMSG_EXEC_COMMAND);
            return false;
        });
    };

    smq.subscribe("self", SMSG_COMMAND_LIST,{
        "datatype": "json", "onmsg": commandList
    });


    //Remove non usable trailing chars in the b64 encoded unique device ID
    function cleanDkey(dkey) {
        return dkey.replace(/\+/g, '-').replace(/\//g, '_').replace(/\=+$/, '');
    };

    function devInfo(info, ptid) {
        if(lightBanks++ == 0) {
	    $('body').addClass('lights-loaded');
            $("#devices").html('<tr><td></td><td></td><td></td><td></td></tr>');
        }
        //Loop over all LEDS and create a TR element for each LED (Ref-TR)
        var lights=info.lights;
        var devTid=info.etid; //Light Controller's (device's) ephemeral topic ID
        var html="";

        var dkey=cleanDkey(info.dkey);
        var first=true;

        for(var i=0 ; i < lights.length; i++) {
            // TR contains: TD for name + TD for LED + TD for LED on/off switch
            if(lights[i].enabled) {
                html+='<tr class="dev-'+dkey+'"'
                if(first) {
                    first=false;
                    html += ' prio="'+info.priority+'"';
                }
                html += '>'+mkLight(devTid, i+1, lights[i].color,lights[i].state) +
					mkLightName(lights[i].name) +
                    mkLightSwitch(devTid, i+1, lights[i].state)+
                    '</tr>';
            }
        }
        $("#commands").show();
        $("#devices .dev-"+dkey).remove(); // remove old, if any

        var insert=true;
        $("#devices tr[prio]").each(function() {
            var prio = $(this).attr("prio")
            if(insert && info.priority < prio) {
                insert=false;
                $(this).before(html);
            }
        });
        if(insert)
            $("#devices").append(html);
        $("#devices").find(".dev-"+dkey+" :checkbox").click(function(ev) {
            var id = $(this).prop('id').match(/(\d+)-(\d+)/);
            var devTid = parseInt(id[1]);
            var ledId = parseInt(id[2]);
            var data = new Uint8Array(2);
            data[0] = ledId;
            data[1] = this.checked ? 1 : 0;
            smq.publish(data,devTid,SMSG_SET_LIGHT);
        });
        
    };
    var infoT={ "datatype": "json", "onmsg": devInfo };
    smq.subscribe("self", SMSG_DEVICE_INFO, infoT);
    smq.subscribe("MSG_DEVICE_INFO", SMSG_DEVICE_INFO, infoT);

    function disconnect(devT) {
        var dkey=cleanDkey(devT.dkey);
        $(".dev-"+dkey).remove(); // remove from browser
        if($("#devices tr").length <= 1) {
            $("#commands").hide();
            smq.onconnect(); //Show message
        }
    };
    smq.subscribe("MSG_DEVICE_DISCONNECT",
                  {"datatype": "json", "onmsg": disconnect});

    //When a device publishes light state change.
    function lightState(data, ptid) {
        var ledId='#switch-'+ptid+'-'+data[0];
        var checked = data[1] ? true : false;
        $(ledId).prop('checked',checked);
        ledId ='#led-'+ptid+data[0];
        if(checked)
            $(ledId).removeClass('led-off')
        else
            $(ledId).addClass('led-off');
        $("#switch")[0].play();
    };
    smq.subscribe("MSG_LIGHT_STATE", {"onmsg":lightState});

    var ut=$.cookie("LightControllerUser");
    if(ut) {
        $("#login").html('<a href="'+(ut=='admin' ? 'admin/' : 'config/')+'">Login</a>');
    }
});
