<?lsp

-- We get cfgT and domain from ENV (set in index.lsp)
local cfgT,domain = cfgT,domain

local colorChoices={"white", "green",  "yellow", "red", "blue"}

local function set(devT,lix,key,val)
   if lix then
      local lightT = devT.lightsT[tonumber(lix)]
      if lightT then lightT[key]=app.xssfilter(val) end 
   end
end

if request:method() == "POST" then
   local b = app.getBroker(domain)
   if data.devtst then -- AJAX via Jquery: Test button clicked
      local dkey, lightIx = data.devtst:match("([^:]-):(%d)")
      for _,peerT in b.peers() do
         if peerT.dkey == dkey then
            ba.timer(function()
                        local toggle=true
                        for i=1,8 do
                           app.pubSetLight(b, peerT.tid, lightIx, toggle)
                           toggle = not toggle
                           coroutine.yield(true)
                        end
                     end):set(1000,true,true)
            break
         end
      end
      response:setstatus(204)
      return;
   end
   if data.terminatebut == "Terminate Service" then
      app.removeDomain(data.d)
      response:sendredirect"./"
   end
   local devT = cfgT.devices[data.dkey]
   if devT then
      if data.savebut == "Save" then
         for _,lightT in ipairs(devT.lightsT) do
            lightT.enabled=false
         end
         for k,v in request:datapairs() do
            set(devT,k:match("enable:(%d)"),"enabled",true)
            set(devT,k:match("name:(%d)"),"name",v)
            set(devT,k:match("color:(%d)"),"color",v)
         end
         devT.priority = tonumber(data.priority) or devT.priority

         -- Make sure we have no conflicting priorities
         local dT1 = devT
         local stop
         repeat
            stop=true
            for _,dT2 in pairs(cfgT.devices) do
               if dT1.priority == dT2.priority and dT1 ~= dT2 then
                  dT2.priority = dT2.priority+1
                  dT1=dT2
                  stop=false
                  break
               end
            end
         until stop

         app.cfgfDomain(domain,cfgT)
         for _,peerT in b.peers() do
            if peerT.dkey == data.dkey then
               b.sm:updateCfg(peerT,cfgT)
               app.pubDeviceInfo(domain,b,peerT,cfgT)
               break
            end
         end
      elseif data.deletebut == "Delete" then
         if not devT.active then
            cfgT.devices[data.dkey]=nil
            app.cfgfDomain(domain,cfgT)
         end
      elseif data.resetbut == "Reset" then

         for _,peerT in b.peers() do
            if peerT.dkey == data.dkey then
               b.shutdown("Forced reset by administrator.", peerT.tid)
               cfgT.devices[data.dkey]=nil
               app.cfgfDomain(domain,cfgT)
               break
            end
         end
      end
      response:sendredirect(request:url().."?d="..domain)
   elseif data.maxconbut == "Save" then
      local maxcon = data.maxcon and tonumber(data.maxcon)
      if maxcon then
         cfgT.maxDevs = maxcon
         app.cfgfDomain(domain,cfgT)
      end
   end
elseif data.updateTime then
   local b = app.getBroker(domain)
   response:json{updateTime=b.updateTime}
end

-- Get broker (by name) and call broker.peers()
-- https://realtimelogic.com/ba/doc/?url=lua/SMQ.html#peers
for _,peerT in app.getBroker(domain).peers() do
   if peerT.dkey then -- if a device (dkey set in .preload)
      local devT = cfgT.devices[peerT.dkey]
      if devT then devT.active=true end -- Used below: display act./inact.
   end
end
?>
<!doctype html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link type="text/css" href="../LightController.css" rel="stylesheet">
    <link type="text/css" href="../domain.css" rel="stylesheet">
  <script type='text/javascript' src='/rtl/jquery.js'></script> 
  <script type='text/javascript' src='../domain-js.lsp?d=<?lsp=domain?>'></script> 
</head>

<body class="logged-in-page domain-page">
<div class="container">
<div class="content-wrap">

<h1 class="page-title">Light Controllers</h1>

<p class="breadcrumb"><a href="../">Light Controller</a> > <a href="./"><?lsp=isUserConfig and "Configure" or "Admin"?></a> > <?lsp=isUserConfig and "Advanced" or domain?></p>
   
<?lsp
local atLeastOne=false
local banksT={}
for dkey,devT in pairs(cfgT.devices) do
   atLeastOne=true
   local bankT={}
   local function wrt(...) table.insert(bankT, table.concat({...})) end
   wrt('<form method="post"><fieldset class="',
       devT.active and 'fsActive' or 'fsInactive',
       '"><legend>&nbsp;',
       devT.name,': ',devT.active and '' or 'in',
       'active &nbsp;</legend><table>',
       '<tr><th>Enable</th><th>Name</th><th>Color</th><th>Test</th>')
   for i,lightT in ipairs(devT.lightsT) do
      wrt('<tr>',
          '<td><input type="checkbox" name="enable:',i,'"',
          lightT.enabled and ' checked' or '','></td>',
          '<td><input type="text" name="name:',i,'" value="',
          lightT.name,'"></td><td><select name="color:',i,'">')
      for _,color in ipairs(colorChoices) do
         wrt('<option value="',color,'"',
             color == lightT.color and ' selected="selected"' or '',
             '>',color,'</option>')
      end
      wrt'</select></td>'
      if devT.active then
         wrt('<td><a href="#" title="Test light ',i,
             '" class="tstbtn" key="',dkey,':',i,'"></a></td>')
      else
         wrt('<td></td>')
      end
      wrt'</tr>'
   end
   wrt'</table>'
   wrt('<div class="prio"><b>Priority:</b> ',
       '<input type="range" min="0" max="30" class="priority" name="priority" value="',
       devT.priority,'" /></div>')
   wrt('<input type="hidden" name="dkey" value="',dkey,
       '"><input type="submit" name="savebut" value="Save" />')
   if devT.active then
      wrt'<input class="rightbut" type="submit" name="resetbut" title="Send reset and force device into Access Point Mode" value="Reset" />'
      local uptime = os.time() - devT.connectTime -- In seconds
      wrt(string.format("<p> Uptime: %02d : %02d</p>",
                        uptime//3600, (uptime//60) % 60))
   else
      wrt'<input class="rightbut" type="submit" name="deletebut" value="Delete" />'
   end
   wrt'</fieldset></form>'
   table.insert(banksT, {priority=devT.priority, html=table.concat(bankT)})
end
if atLeastOne then
   -- Sort Lua table (list of light banks) based on the light bank's priority.
   local function listComp(dsm1,dsm2)
      return dsm1.priority < dsm2.priority
   end
   table.sort(banksT, listComp)
   for _, bankT in pairs(banksT) do
      response:write(bankT.html)
   end
else
?>
<p class="alert-no-conn">You do not have any light controller devices connected to the domain <span>"<?lsp=domain?>"!</span></p>
<?lsp end ?>
<?lsp if not isUserConfig then ?>
   <ul id="dlinks">
   <li><a target="_blank" href="http://<?lsp=domain?>/config/">Home user's configuration page</a></li>
   <li><a target="_blank" href="http://<?lsp=domain?>/">Light Controller</a></li>
   </ul>
<?lsp end ?>
  <fieldset id="securityfield">
    <legend>Security</legend>
    <form method="POST">
      Maximum number of connected Light Controllers: <input type="number" name="maxcon" value="<?lsp=cfgT.maxDevs?>"/>
    	<input type="submit" name="maxconbut" value="Save" />
      
    </form>
  </fieldset>
<?lsp if not isUserConfig then ?>
  <form method="post" id="terminate_service_form">
  <fieldset>
    <legend></legend>
	<div class="button">
    	<input id="termbut" type="submit" name="terminatebut" value="Terminate Service" />
	</div>
  </fieldset>
</form>
<?lsp end ?>
</div>
<div class="footer">
  <a target="help" id="helpb" href="../doc/manual.html#admcfg"><img src="../images/help.png" alt="Help" /></a>
  <a href="../doc/"><img src="../images/footer-logo.png" alt="Real Time Logic" /></a>
</div>
</div>
<audio id="ping"><source src="../audio/ping.mp3" type="audio/mp3" /></audio>
<audio id="pong"><source src="../audio/pong.mp3" type="audio/mp3" /></audio>
</body>
</html>
